package com.wgmf.library.location

import android.content.Context
import android.location.Location
import android.os.Looper
import com.huawei.hmf.tasks.OnSuccessListener
import com.huawei.hms.location.*
import com.urovo.wugumofang.constants.LocationParams
import com.urovo.wugumofang.system.LocationUtil
import com.zsy.libcore.common.android.log.Log
import java.util.*

/**
 * 华为融合定位
 * @author LW
 */
class HwLocationManager(
    private val context: Context?,
    private val callback: LocationUtil.OnCallback?
) {
    private val mSettingsClient = LocationServices.getSettingsClient(context)

    private val mSystemLocationManager = SystemLocationManager(context, callback)

    // 定位交互接入对象
    private val mFusedLocationProviderClient: FusedLocationProviderClient =
        LocationServices.getFusedLocationProviderClient(context)

    private val mLocationRequest: LocationRequest by lazy {
        LocationRequest().apply {
            priority = LocationRequest.PRIORITY_HIGH_ACCURACY
            numUpdates = 1
        }
    }

    /**
     * 校验定位状态，并启动定位
     */
    fun startLocation() {
        val builder = LocationSettingsRequest.Builder()
        builder.addLocationRequest(mLocationRequest)

        val locationSettingsRequest = builder.build()
        // 检查设备定位设置
        mSettingsClient.checkLocationSettings(locationSettingsRequest)
            // 检查设备定位设置接口调用成功监听
            .addOnSuccessListener { locationSettingsResponse ->
                val locationSettingsStates = locationSettingsResponse.locationSettingsStates
                val isLocationUsable = locationSettingsStates.isLocationUsable
                val isHMSLocationUsable = locationSettingsStates.isHMSLocationUsable
                val isNetworkLocationUsable = locationSettingsStates.isNetworkLocationUsable
                val isGnssUsable = locationSettingsStates.isGnssUsable
                val isBleUsable = locationSettingsStates.isBleUsable

                val stringBuilder = StringBuilder()
                // 定位开关是否打开
                stringBuilder.append("isLocationUsable=")
                    .append(isLocationUsable)
                // HMS Core是否可用
                stringBuilder.append(",isHMSLocationUsable=")
                    .append(isHMSLocationUsable)
                //网络位置服务开关是否打开
                stringBuilder.append(",isNetworkLocationUsable=")
                    .append(isNetworkLocationUsable)
                //GNSS位置服务能力是否可用
                stringBuilder.append(",isGnssUsable=")
                    .append(isGnssUsable)
                //蓝牙开关是否打开
                stringBuilder.append(",isBleUsable=")
                    .append(isBleUsable)
                requestLocationUpdates()
                // 打印详细地址信息
                Log.i("Amap", "checkLocationSetting onComplete:$stringBuilder")
            }
            // 检查设备定位设置接口失败监听回调
            .addOnFailureListener { e ->
                Log.i("Amap", "checkLocationSetting onFailure:" + e.message)
                //getLastLocation()
                mSystemLocationManager.startLocation()
            }
    }

    private val mLocationCallback: LocationCallback = object : LocationCallback() {
        override fun onLocationResult(locationResult: LocationResult?) {
            locationResult?.let {
                val locations = it.locations
                if (locations.isNotEmpty()) {
                    getGeocoder(locations[0])
                }
            }
        }
    }

    /**
     * 最后一次定位信息
     */
    private fun getLastLocation() {
        val lastLocation =
            mFusedLocationProviderClient.lastLocation
        lastLocation.addOnSuccessListener(OnSuccessListener { location ->
            if (location != null) {
                getGeocoder(location)
            } else {
                mSystemLocationManager.startLocation()
            }
            return@OnSuccessListener
        })
            // 获取最后的已知位置失败监听回调
            .addOnFailureListener {
                mSystemLocationManager.startLocation()
            }
    }

    private val mTimer = Timer()

    /**
     * 开始定位
     */
    fun requestLocationUpdates() {
        mFusedLocationProviderClient.requestLocationUpdates(
            mLocationRequest,
            mLocationCallback,
            Looper.getMainLooper()
        ).addOnSuccessListener {
            Log.i("Amap", "请求定位成功")
            mTimer.schedule(object : TimerTask() {
                override fun run() {
                    //华为融合定位无法获取到数据，启动系统定位获取
                    if (LocationParams.Longitude.isNullOrEmpty() || LocationParams.Latitude.isNullOrEmpty()) {
                        mSystemLocationManager.startLocation()
                    }
                    mTimer.cancel()
                }
            }, 8000)
        }
            .addOnFailureListener {
                Log.i("Amap", "请求定位失败")
                getLastLocation()
            }
    }

    /**
     * 关闭定位
     */
    fun closeLocation() {
        // 注意：停止位置更新时，mLocationCallback必须与requestLocationUpdates方法中的LocationCallback参数为同一对象。
        mFusedLocationProviderClient.removeLocationUpdates(mLocationCallback)
            // 停止位置更新成功监听回调
            .addOnSuccessListener {
                Log.i("Amap", "停止位置更新成功的回调")
            }
            // 停止位置更新失败监听回调
            .addOnFailureListener {
                Log.i("Amap", "停止位置更新失败的回调")
            }

        mSystemLocationManager.closeLocation()

        mTimer.cancel()
    }

    private fun getGeocoder(location: Location) {
        Log.i("Amap", "华为融合定位")
        LocationUtil.getGeocoder(context, location, callback)
    }
}