package com.urovo.wugumofang.utils

import android.os.Handler
import android.os.Looper
import org.jetbrains.anko.collections.forEachReversedByIndex

/**
 * @author Sasuke on 2019-12-3 0003.
 * 任务队列支配者 （自定义任务队列，解决回调地狱）
 *
 * 使用方法：
var dominator = TaskEnqueueDominator().addCase(...)
.addCase(...)
.addPauseCase()//添加一个需要悬停的任务，后面的任务会暂停执行, 调用go()重新执行队列
.addCase(...)
.go()

enqueDominator.go()//继续执行剩余未完成的队列（一般搭配Pause类型的具体Case使用）
在其它地方如弹窗的点击监听、网络请求的回调处，需要继续执行任何队列里的剩余任务时，go()一下

 * 注：所有元素都从0位插入，遍历的时候也从尾部开始，这样才能一边循环，一边删除
 * 注：后来我才知道Java还自带一个Queue这个队列类。。。
 */
class TaskEnqueueDominator {
    val taskList = mutableListOf<BaseCase>()

    abstract class BaseCase {
        var enqueDominator: TaskEnqueueDominator? = null
        open var pause = false//默认是不等待的，一口气顺序执行。

        abstract fun dowork()

        fun doInMainThread(tempFun: (() ->Unit)?){
            Handler(Looper.getMainLooper()).post { tempFun?.invoke() }
        }
    }

    fun isEmpty(): Boolean {
        return taskList.isNullOrEmpty()
    }

    fun isNotEmpty(): Boolean{
        return !isEmpty()
    }

    /**添加任务，并且该任务完成后，继续执行队列后面的任务*/
    fun addCase(case: BaseCase): TaskEnqueueDominator {
        taskList.run { add(0, case.apply { pause = false;enqueDominator = this@TaskEnqueueDominator }) }
        return this
    }

    /**添加任务，并且该任务完成后，暂停！不执行后面的任务*/
    fun addPauseCase(case: BaseCase): TaskEnqueueDominator {
        taskList.run { add(0, case.apply { pause = true; enqueDominator = this@TaskEnqueueDominator }) }
        return this
    }

    fun go(): TaskEnqueueDominator {
        run taskEnqueue@{
            taskList.forEachReversedByIndex {
                val pause = it.pause
                taskList.remove(it)//这个一定要先remove，因为dowork()可能会有dominator.go()会导致又执行新的循环。
                it.dowork()

                if (pause) {
                    return@taskEnqueue
                }
            }
        }
        return this
    }

}

