package com.zsy.libcore.model.data.net.helper;

import com.zsy.libcore.config.LibCoreConfig;
import com.zsy.libcore.model.data.net.security.HttpsTrustManager;

import java.io.IOException;
import java.util.HashMap;
import java.util.concurrent.TimeUnit;

import okhttp3.Interceptor;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.logging.HttpLoggingInterceptor;
import retrofit2.Retrofit;
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory;
import retrofit2.converter.gson.GsonConverterFactory;


/**
 * addInterceptor(new LocalCacheInterceptor(maxCacheSeconds, commonHeaders))
 * addNetworkInterceptor(new NetCacheInterceptor(maxCacheSeconds, commonHeaders)).cache(OKHTTP_LOCAL_CACHE);
 */
public class RetrofitBaseHelper {
    public static int DEFAULT_TIMEOUT = 30;//默认30秒超时

    //以不同的超时时长为key，把不同超时时长的retrofit缓存到HashMap里。
    //因为这些基于okhttp的都是单例的，所以要想针对不同的请求设置不同的超时，那就得生成多个retrofit实例。
    private static HashMap<Integer, Retrofit> timeoutRetrofits = new HashMap<>();

    public static Retrofit createRetrofit(final String apiUrl, final HashMap<String, String> myHeaders, int timeout) {
        OkHttpClient.Builder httpClientBuilder = getHttpClientBuilder(myHeaders, timeout);

        Retrofit.Builder builder = new Retrofit.Builder();
        if (apiUrl.startsWith("https://")) {
            builder.client(HttpsTrustManager.getUnsafeOkHttpClient(httpClientBuilder).build());
        } else {
            builder.client(httpClientBuilder.build());
        }
        builder.baseUrl(apiUrl)
                .addConverterFactory(GsonConverterFactory.create())
                .addCallAdapterFactory(RxJava2CallAdapterFactory.create());
        //做到这一步其实还没有缓存，返回到外面再做判断，如果不是默认超时时长，那就缓存到HashMap里。
        return builder.build();
    }

    /**
     * @param headers 头部参数
     * @param timeout 超时时间
     */
    private static OkHttpClient.Builder getHttpClientBuilder(HashMap<String, String> headers, int timeout) {
        /**方式一*/
        //OkHttpClient okHttpClient = new OkHttpClient();
        //okHttpClient.interceptors().add(new LocalCacheInterceptor());
        //okHttpClient.interceptors().add(new NetCacheInterceptor());
        //okHttpClient.interceptors().add(new LoggingInterceptor());//自定义的
        //okHttpClient.interceptors().add(new HttpLoggingInterceptor());//用okhttp3自带的

        OkHttpClient.Builder httpClientBuilder = new OkHttpClient.Builder();

        //int cacheSize = 10 * 1024 * 1024; // 10 MiB
        //File cacheDir = new File(LibCoreApp.getInstance().getCacheDir(), "cache.json");
        //File cacheDir = FileUtil.getSaveFile(LibCoreApp.PACKAGE_NAME,"cache.json");
        //File cacheDir = new File(LibCoreConstants.Path.PATH_CACHE);
        //Cache cache = new Cache(cacheDir, cacheSize);

        if (LibCoreConfig.IS_DEBUG_ENABLE) {
            HttpLoggingInterceptor httpLoggingInterceptor = new HttpLoggingInterceptor();
            httpLoggingInterceptor.setLevel(HttpLoggingInterceptor.Level.BODY);
            httpClientBuilder.addInterceptor(httpLoggingInterceptor);
        } else {
            HttpLoggingInterceptor httpLoggingInterceptor = new HttpLoggingInterceptor();
            httpLoggingInterceptor.setLevel(HttpLoggingInterceptor.Level.HEADERS);
            httpClientBuilder.addInterceptor(httpLoggingInterceptor);
        }
        //设置缓存;这里跟java通用性处理,就不添加拦截器
        //httpClientBuilder.addNetworkInterceptor(new NetCacheInterceptor());
        //httpClientBuilder.cache(cache);
        //设置超时
        httpClientBuilder.connectTimeout(timeout, TimeUnit.SECONDS);
        httpClientBuilder.readTimeout(timeout, TimeUnit.SECONDS);
        httpClientBuilder.writeTimeout(timeout, TimeUnit.SECONDS);
        //错误重连
        //httpClientBuilder.retryOnConnectionFailure(true);

        //加自定义的头
        if (!headers.isEmpty()) {
            httpClientBuilder.addInterceptor(new Interceptor() {
                @Override
                public Response intercept(Chain chain) throws IOException {
                    Request original = chain.request();
                    Request.Builder requestBuilder = original.newBuilder().method(original.method(), original.body());
                    //把头加进去
                    for (String key : headers.keySet()) {
                        requestBuilder.addHeader(key, headers.get(key));
                    }
                    return chain.proceed(requestBuilder.build());
                }
            });
        }
        return httpClientBuilder;
    }

    private static Retrofit singleton;

    /**
     * 创建一个默认超时时长的createApi（保存到全局单例）
     */
    public static <T> T createApi(String apiUrl, Class<T> clazz, final HashMap<String, String> myHeaders) {
        if (singleton == null) {
            synchronized (RetrofitBaseHelper.class) {
                if (singleton == null) {
                    singleton = createRetrofit(apiUrl, myHeaders, DEFAULT_TIMEOUT);//默认超时时长。
                }
            }
        }
        return singleton.create(clazz);
    }

    private static Retrofit newSingleton;

    /**
     * 创建一个默认超时时长的createApi（保存到全局单例）
     */
    public static <T> T createNewApi(String apiUrl, Class<T> clazz, final HashMap<String, String> headers) {
        if (newSingleton == null) {
            synchronized (RetrofitBaseHelper.class) {
                if (newSingleton == null) {
                    newSingleton = createRetrofit(apiUrl, headers, DEFAULT_TIMEOUT);//默认超时时长。
                }
            }
        }
        return newSingleton.create(clazz);
    }

    /**
     * 创建一个自定义超时时长的createApi（保存到HashMap）
     */
    public static <T> T createApiTimeout(String apiUrl, Class<T> clazz, final HashMap<String, String> headers, int timeout) {
        //先在HashMap里找找看有没有，有缓存好的就直接拿来用。没找到？那就重新创建一个新的。
        if (timeoutRetrofits.containsKey(timeout)) {
            return timeoutRetrofits.get(timeout).create(clazz);
        }
        synchronized (RetrofitBaseHelper.class) {
            if (timeoutRetrofits.containsKey(timeout)) {
                return timeoutRetrofits.get(timeout).create(clazz);
            }
            //如果没缓存，那就现场造一个，并缓存起来。
            Retrofit retrofit = createRetrofit(apiUrl, headers, timeout);
            timeoutRetrofits.put(timeout, retrofit);
            return retrofit.create(clazz);
        }
    }
}
