package com.zsy.libcore.ui.ada;

import android.animation.Animator;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.view.animation.Interpolator;
import android.view.animation.LinearInterpolator;

import com.zsy.libcore.listener.OnItemClickListener;
import com.zsy.libcore.ui.anim.AnimHelper;

import java.util.List;

import androidx.recyclerview.widget.RecyclerView;

/**
 * RecyclerView适配器基类
 *
 * @param <E> 数据类型
 * @param <VH> ViewHolder类型
 * samy(hongfeiliuxing@gmail.com)
 * @datetime 2015-09-04 19:53
 */
public abstract class BaseRecyclerViewAda<E, VH extends RecyclerView.ViewHolder> extends RecyclerView.Adapter<VH> {
	// 用传递过来的上下文靠谱点；而不是viewGroup.getContext()
	protected Context mContext;
	protected LayoutInflater mInflater;
	protected List<E> mDataSet;
	// private Map<Integer, OnItemClickListener<E>> mOnItemClickListener;
	// protected OnItemClickListener mOnItemClickListener;
	OnItemClickListener<E> mItemClickListener;

	/**
	 * 添加动画和变换样式部分
	 */
	private int mDuration = 300;
	private Interpolator mInterpolator = new LinearInterpolator();
	private boolean isFirstOnly = true;
	private int mLastPosition = -1;

	public BaseRecyclerViewAda(Context mContext, List<E> dataSet) {
		this.mContext = mContext;
		this.mDataSet = dataSet;
		mInflater = LayoutInflater.from(mContext);
	}

	public E getItem(int position) {
		/** 防止外部调用时越界 */
		if (mDataSet == null || position < 0 || position > mDataSet.size()) return null;
		return mDataSet.get(position);
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	@Override
	public int getItemCount() {
		return (mDataSet == null) ? 0 : mDataSet.size();
	}

	public List<E> getList() {
		return mDataSet;
	}

	public void setList(List<E> items) {
		if (items != null) {
			mDataSet.clear();
		}
		mDataSet.addAll(items);
		notifyDataSetChanged();
	}

	public void addItems(List<E> items) {
		if (items != null) {
			mDataSet.addAll(items);
			notifyDataSetChanged();
		}
	}

	public void addItem(E e) {
		mDataSet.add(0, e);
		notifyItemInserted(0);
	}

	public void removeItem(E e) {
		mDataSet.remove(e);
		notifyDataSetChanged();
	}

	public void removeList(int position) {
		mDataSet.remove(position);
		notifyDataSetChanged();
		// notifyItemRemoved(position);
	}

	protected View inflateItemView(ViewGroup viewGroup, int layoutId) {
		return inflateItemView(viewGroup, layoutId, false);
	}

	protected View inflateItemView(ViewGroup viewGroup, int layoutId, boolean attach) {
		// return LayoutInflater.from(viewGroup.getContext()).inflate(layoutId, viewGroup, attach);
		return mInflater.inflate(layoutId, viewGroup, attach);
	}

	@Override
	public final void onBindViewHolder(VH viewHolder, int position) {
		final E item = getItem(position);
		bindDataToItemView(viewHolder, item);
		setupItemViewClickListener(viewHolder, position, item);
		// 后期呆优化这里的点击事件
		// setViewClickListener(viewHolder.itemView, position, mDataSet.get(position));
	}

	// private void addInternalClickListener(final View itemV, final Integer position, final E valuesMap) {
	// if (mOnItemClickListener != null) {
	// for (Integer key : mOnItemClickListener.keySet()) {
	// View inView = itemV.findViewById(key);
	// final OnItemClickListener<E> listener = mOnItemClickListener.get(key);
	// if (inView != null && listener != null) {
	// inView.setOnClickListener(new OnClickListener() {
	// @Override
	// public void onClick(View view) {
	// listener.OnClickListener(itemV, view, position, valuesMap);
	// }
	// });
	// }
	// }
	// }
	// }

	// protected void setViewClickListener(final View itemView, final Integer position, final E item) {
	// if (mOnItemClickListener != null) {
	// itemView.setOnClickListener(new OnClickListener() {
	// @Override
	// public void onClick(View view) {
	// mOnItemClickListener.OnClickListener(itemView, view, position, item);
	// }
	// });
	// }
	// }

	public void setOnItemClickListener(OnItemClickListener<E> mItemClickListener) {
		this.mItemClickListener = mItemClickListener;
	}

	/**
	 * ItemView的点击事件
	 */
	protected void setupItemViewClickListener(VH viewHolder, final int position, final E item) {
		viewHolder.itemView.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				if (mItemClickListener != null) {
					mItemClickListener.onClick(position, item);
				}
			}
		});
	}

	// /**
	// * 用map处理防止多次设置监听处理,如果麻烦的话，可以直接的子类初始化控件设置监听,这里做的是公共的点击事件
	// *
	// * @param key
	// * @param onClickListener
	// */
	// public void setOnItemClickListener(Integer key, OnItemClickListener<E> onClickListener) {
	// if (mOnItemClickListener == null) {
	// mOnItemClickListener = new HashMap<>();
	// }
	// mOnItemClickListener.put(key, onClickListener);
	// }

	protected abstract void bindDataToItemView(VH viewHolder, E item);

	/// **
	// * 内部类监听;做的是公共的点击事件
	// *
	// * @param <E>
	// */
	// public interface OnItemClickListener<E> {
	// // void OnClickListener(VH viewHolder, E item);
	// void OnClickListener(View parentV, View v, int position, E value);
	// }

	public void setDuration(int duration) {
		mDuration = duration;
	}

	public void setInterpolator(Interpolator interpolator) {
		mInterpolator = interpolator;
	}

	public void setFirstOnly(boolean firstOnly) {
		isFirstOnly = firstOnly;
	}

	protected void animate(RecyclerView.ViewHolder holder, int position) {
		if (!isFirstOnly || position > mLastPosition) {
			for (Animator anim : getAnimators(holder.itemView)) {
				anim.setDuration(mDuration).start();
				anim.setInterpolator(mInterpolator);
			}
			mLastPosition = position;
		}
		else {
			AnimHelper.clear(holder.itemView);
		}
	}

	// protected abstract Animator[] getAnimators(View view);
	protected Animator[] getAnimators(View view) {
		return new Animator[0];
	}
}
