package com.urovo.activity.goodsmanager.adapter.order

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.RelativeLayout
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.RecyclerView
import com.urovo.activity.goodsmanager.model.OrderDeliveryDifferenceQueryEntity
import com.urovo.wugumofang.R
import com.urovo.wugumofang.data.StringUtils
import com.urovo.wugumofang.utils.TextColorDominator
import com.urovo.wugumofang.utils.convertDefaultValue
import com.urovo.wugumofang.utils.isNotNullOrSize
import com.urovo.wugumofang.view.scrollablePanel.PanelAdapter

/**
 * @author LW on 2023-12-1 0001.
 * 订货与差异适配器表格
 */
class OrderDeliveryDifferenceTableAdapter(private val mContext: Context) : PanelAdapter {

    /**
     * 订货列表
     */
    private var mDiffDetailsList: MutableList<OrderDeliveryDifferenceQueryEntity.DiffDetailsBean> =
        mutableListOf()

    /**
     * 表单每列标题
     */
    private var mTableTitleList: List<String> = listOf()

    companion object {
        /**
         * 列标题类型
         */
        const val COLUMNS_TITLE_ITEM_TYPE: Int = 1

        /**
         * 行标题类型
         */
        const val ROW_TITLE_ITEM_TYPE: Int = 2

        /**
         * 行列标题类型
         */
        const val ROW_COLUMNS_TITLE_ITEM_TYPE: Int = 3

        /**
         * 内容类型
         */
        const val CONTENT_ITEM_TYPE: Int = 0
    }

    override fun getRowCount(): Int = mDiffDetailsList.size

    override fun getColumnCount(): Int = mTableTitleList.size

    override fun getItemViewType(row: Int, column: Int): Int {
        if (row == 0 && column == 0) {
            return ROW_COLUMNS_TITLE_ITEM_TYPE
        }
        if (row == 0) {
            return ROW_TITLE_ITEM_TYPE
        }
        if (column == 0) {
            return COLUMNS_TITLE_ITEM_TYPE
        }
        return CONTENT_ITEM_TYPE
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder?, row: Int, column: Int) {
        when (getItemViewType(row, column)) {
            ROW_COLUMNS_TITLE_ITEM_TYPE -> {
                bindRowColumnTitleItemData(holder as RowColumnsTitleViewHolder, column)
            }
            ROW_TITLE_ITEM_TYPE -> {
                bindRowTitleItemData(holder as RowTitleViewHolder, column)
            }
            COLUMNS_TITLE_ITEM_TYPE -> {
                bindColumnTitleItemData(holder as ColumnsTitleViewHolder, row)
            }
            else -> {
                bindContentItemData(holder as ContentViewHolder, row, column)
            }
        }
    }

    private fun bindContentItemData(
        contentViewHolder: ContentViewHolder,
        row: Int,
        column: Int
    ) {
        if (mDiffDetailsList.isNotNullOrSize()) {
            val diffDetails = mDiffDetailsList[row - 1]
            diffDetails.apply {
                contentViewHolder.valueTv.text = when (column) {
                    1 -> diffQty.toString().convertDefaultValue()
                    2 -> deliveryQty.toString().convertDefaultValue()
                    3 -> auditQty.toString().convertDefaultValue()
                    4 -> orderQty.toString().convertDefaultValue()
                    else -> "".convertDefaultValue()
                }
                updateItemBgColor(row, contentViewHolder.itemRootLl)
            }
        }
    }

    private fun bindColumnTitleItemData(
        titleViewHolder: ColumnsTitleViewHolder,
        row: Int
    ) {
        if (mDiffDetailsList.isNotNullOrSize()) {
            val diffDetails = mDiffDetailsList[row - 1]
            titleViewHolder.columnsTitleTv.text = diffDetails.proName.convertDefaultValue()
            val content = StringUtils
                .formatString(
                    mContext.getString(R.string.order_delivery_difference_proNo),
                    diffDetails.proNo.convertDefaultValue()
                ) + diffDetails.proUOM
            titleViewHolder.columnsInfoTv.text =
                TextColorDominator.styleTextColor(
                    content, ContextCompat.getColor(
                        mContext,
                        R.color.color_common_red_one
                    ),
                    content.lastIndexOf(")")+1, content.length
                )
            updateItemBgColor(row, titleViewHolder.itemRootLl)
        }
    }

    private fun updateItemBgColor(position: Int, item: View) {
        if (position % 2 == 0) {
            item.setBackgroundColor(
                ContextCompat.getColor(
                    mContext,
                    R.color.color_FFFFFF
                )
            )
        } else {
            item.setBackgroundColor(
                ContextCompat.getColor(
                    mContext,
                    R.color.color_F7F7F7
                )
            )
        }
    }

    private fun bindRowTitleItemData(
        titleViewHolder: RowTitleViewHolder,
        column: Int
    ) {
        if (mTableTitleList.isNotNullOrSize()) {
            titleViewHolder.rowTitleTv.text = mTableTitleList[column]
        }
    }

    private fun bindRowColumnTitleItemData(
        titleViewHolder: RowColumnsTitleViewHolder,
        column: Int
    ) {
        if (mTableTitleList.isNotNullOrSize()) {
            titleViewHolder.rowColumnsTitleTv.text = mTableTitleList[column]
        }
    }


    override fun onCreateViewHolder(parent: ViewGroup?, viewType: Int): RecyclerView.ViewHolder {
        when (viewType) {
            ROW_COLUMNS_TITLE_ITEM_TYPE -> {
                return RowColumnsTitleViewHolder(
                    LayoutInflater.from(
                        parent?.context
                    ).inflate(
                        R.layout.item_order_delivery_difference_table_row_columns_title,
                        parent,
                        false
                    )
                )
            }
            ROW_TITLE_ITEM_TYPE -> {
                return RowTitleViewHolder(
                    LayoutInflater.from(
                        parent?.context
                    ).inflate(
                        R.layout.item_order_delivery_difference_table_row_title,
                        parent,
                        false
                    )
                )
            }
            COLUMNS_TITLE_ITEM_TYPE -> {
                return ColumnsTitleViewHolder(
                    LayoutInflater.from(
                        parent?.context
                    ).inflate(
                        R.layout.item_order_delivery_difference_table_columns_title,
                        parent,
                        false
                    )
                )
            }
            else -> {
                return ContentViewHolder(
                    LayoutInflater.from(
                        parent?.context
                    ).inflate(R.layout.item_order_delivery_difference_table_item, parent, false)
                )
            }
        }
    }

    private class RowTitleViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        val itemRootRl: RelativeLayout =
            view.findViewById(R.id.order_delivery_difference_table_item_root_rl)
        val rowTitleTv: TextView =
            view.findViewById(R.id.order_delivery_difference_table_row_title_tv)
    }

    private class RowColumnsTitleViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        val itemRootRl: RelativeLayout =
            view.findViewById(R.id.order_delivery_difference_table_item_root_rl)
        val rowColumnsTitleTv: TextView =
            view.findViewById(R.id.order_delivery_difference_table_row_columns_title_tv)
    }

    private class ColumnsTitleViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        val itemRootLl: LinearLayout =
            view.findViewById(R.id.order_delivery_difference_table_item_root_ll)
        val columnsTitleTv: TextView =
            view.findViewById(R.id.order_delivery_difference_table_columns_title_tv)
        val columnsInfoTv: TextView =
            view.findViewById(R.id.order_delivery_difference_table_columns_info_tv)
    }

    private class ContentViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        val itemRootLl: LinearLayout =
            view.findViewById(R.id.order_delivery_difference_table_item_root_ll)
        val valueTv: TextView = view.findViewById(R.id.order_delivery_difference_table_value_tv)
    }

    fun setTableTitleList(tableTitleList: List<String>) {
        this.mTableTitleList = tableTitleList
    }

    fun setDiffDetailsList(diffDetailsList: MutableList<OrderDeliveryDifferenceQueryEntity.DiffDetailsBean>) {
        this.mDiffDetailsList = diffDetailsList
    }

    fun getDiffDetailsList(): MutableList<OrderDeliveryDifferenceQueryEntity.DiffDetailsBean> {
        return mDiffDetailsList
    }

}