package com.urovo.activity.membergroup;

import android.animation.ObjectAnimator;
import android.animation.ValueAnimator;
import android.animation.ValueAnimator.AnimatorUpdateListener;
import android.content.Context;
import android.util.AttributeSet;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.ListAdapter;
import android.widget.ListView;

public class ExpandedListView extends ListView {
	private final int DURATION = 500;
	private ObjectAnimator mObjectAnimator;
	private ExpandedListAdapter mExpandedListAdapter;
	private OnSizeChangeListener mOnSizeChangeListener;
	private View mHeaderView;
	private View mExpandView;
	private View mLastHeaderView;
	private View mLastExpandView;
	private int mLastHeight;

	public interface OnSizeChangeListener {
		public void onSizeChange(int height);

		public void onSizeDeltaChange(int delta);
	}

	public ExpandedListView(Context context) {
		super(context);
	}

	public ExpandedListView(Context context, AttributeSet attrs) {
		super(context, attrs);
	}

	public ExpandedListView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
	}

	public void setOnSizeChangeListener(OnSizeChangeListener l) {
		mOnSizeChangeListener = l;
	}

	@Override
	public void setAdapter(ListAdapter adapter) {

		if(!(adapter instanceof ExpandedListAdapter)) {
			throw new IllegalArgumentException("ExpandedListView must need a ExpandedListAdapter!");
		}
		super.setAdapter(adapter);
	}

	public boolean toggleExpandItemByPosition(int position) {
		View v = getChildAt(position);
		mExpandedListAdapter = (ExpandedListAdapter) getAdapter();
		mHeaderView = mExpandedListAdapter.getHeaderView(v);
		mExpandView = mExpandedListAdapter.getContentView(v);
		int height = (Integer) mExpandView.getTag();
		boolean open = true;
		if(null != mHeaderView.getTag()) {
			open = !(Boolean) mHeaderView.getTag();
		}
		expandAnimator(mHeaderView.getTag(), open, height);
		return open;
	}

	public void expandItemByPosition(boolean open, int position) {
		View v = getChildAt(position);
		mExpandedListAdapter = (ExpandedListAdapter) getAdapter();
		mHeaderView = mExpandedListAdapter.getHeaderView(v);
		mExpandView = mExpandedListAdapter.getContentView(v);
		int height = (Integer) mExpandView.getTag();
		expandAnimator(mHeaderView.getTag(), open, height);
	}

	private void expandAnimator(Object lastStatus, boolean open, int height) {
		if(null != mObjectAnimator && mObjectAnimator.isRunning() && null != lastStatus && open != (Boolean) lastStatus) {
			mObjectAnimator.reverse();
		}
		else {
			mObjectAnimator = ObjectAnimator.ofInt(mExpandView, "expand_height", open ? 0 : height, open ? height : 0).setDuration(DURATION);
			mObjectAnimator.addUpdateListener(new AnimatorUpdateListener() {
				@Override
				public void onAnimationUpdate(ValueAnimator animation) {
					int h = (Integer) animation.getAnimatedValue();
					mExpandView.getLayoutParams().height = h;
					mExpandView.requestLayout();
					if(null != mOnSizeChangeListener) {
						mOnSizeChangeListener.onSizeChange(h);
						int delta = h - mLastHeight;
						mOnSizeChangeListener.onSizeDeltaChange(delta);
					}
					mLastHeight = h;
				}
			});
			mObjectAnimator.start();
		}
		mHeaderView.setTag(open);
		mLastHeaderView = mHeaderView;
		mLastExpandView = mExpandView;
	}

	static public abstract class ExpandedListAdapter extends BaseAdapter {
		public final static int ID_HEADER = 100;
		public final static int ID_CONTENT = 101;
		private int mMaxHeight = 200;
		private Context mContext;

		public ExpandedListAdapter(Context cxt) {
			mContext = cxt;
		}

		public final View getHeaderView(View parent) {
			return parent.findViewById(ID_HEADER);
		}

		public final View getContentView(View parent) {
			return parent.findViewById(ID_CONTENT);
		}

		private View createHeader(int position) {
			FrameLayout flHeader;
			flHeader = new FrameLayout(mContext);
			flHeader.setId(ID_HEADER);
			LinearLayout.LayoutParams lpHeader = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
			flHeader.setLayoutParams(lpHeader);
			View header = getHeaderByPosition(null, position);
			if(null != header) {
				flHeader.addView(header);
			}
			return flHeader;
		}

		private void updateHeader(FrameLayout flHeader, int position) {
			View child = flHeader.getChildAt(0);
			child = getHeaderByPosition(child, position);
		}

		private View createContent(int position) {
			FrameLayout flContent;
			flContent = new FrameLayout(mContext);
			flContent.setId(ID_CONTENT);
			LinearLayout.LayoutParams lpContent = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
			flContent.setLayoutParams(lpContent);
			View content = getContentByPosition(null, position);

			if(null != content) {
				flContent.addView(content);
				flContent.measure(MeasureSpec.makeMeasureSpec(0, ViewGroup.LayoutParams.FILL_PARENT), MeasureSpec.makeMeasureSpec(0, ViewGroup.LayoutParams.WRAP_CONTENT));

				int height = flContent.getMeasuredHeight();
				if(height > mMaxHeight)
					height = mMaxHeight;
				flContent.setTag(height);
			}

			flContent.getLayoutParams().height = 0;
			return flContent;
		}

		private void updateContent(FrameLayout flContent, int position) {
			View child = flContent.getChildAt(0);
			child = getContentByPosition(child, position);
			if(null != child) {
				flContent.measure(MeasureSpec.makeMeasureSpec(0, ViewGroup.LayoutParams.FILL_PARENT), MeasureSpec.makeMeasureSpec(0, ViewGroup.LayoutParams.WRAP_CONTENT));
				int height = flContent.getMeasuredHeight();
				if(height > mMaxHeight)
					height = mMaxHeight;
				flContent.setTag(height);
			}
		}

		private View createTemplate(int position) {
			LinearLayout ll = new LinearLayout(mContext);
			LayoutParams lp = new LayoutParams(ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
			ll.setLayoutParams(lp);
			ll.setOrientation(LinearLayout.VERTICAL);

			ll.addView(createHeader(position));
			ll.addView(createContent(position));
			return ll;
		}

		private View updateTemplate(View parent, int position) {
			LinearLayout ll = (LinearLayout) parent;
			FrameLayout flHeader = (FrameLayout) ll.findViewById(ID_HEADER);
			FrameLayout flContent = (FrameLayout) ll.findViewById(ID_CONTENT);
			updateHeader(flHeader, position);
			updateContent(flContent, position);
			return ll;
		}

		@Override
		public final View getView(int position, View convertView, ViewGroup parent) {

			if(null == convertView) {
				convertView = createTemplate(position);
			}
			else {
				convertView = updateTemplate(convertView, position);
			}
			return convertView;
		}

		abstract public View getHeaderByPosition(View convertView, int position);

		abstract public View getContentByPosition(View convertView, int position);

	}
}
