package com.urovo.wgmf.view;

import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.view.Window;
import android.view.animation.Animation;
import android.view.animation.LinearInterpolator;
import android.view.animation.RotateAnimation;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.resource.drawable.GlideDrawable;
import com.bumptech.glide.request.RequestListener;
import com.bumptech.glide.request.target.Target;
import com.urovo.wugumofang.R;

import java.io.Serializable;

/**
 * @author xiep
 */
public class PublicPictureDialog extends Dialog {

    private PublicPictureDialog(Context context, int theme) {
        super(context, theme);
    }

    public interface OnImageReadyCallback {
        void onImageReady(Dialog dialog, String url, ImageView view);
    }

    public static class Builder {
        private Context context;
        private String url;
        private int layoutId;
        private int width = 0, height = 0;
        private OnClickListener positiveButtonClickListener;
        private OnImageReadyCallback imageReadyCallback;
        private int refreshCount = 0;
        private RotateAnimation rotateAnimation;

        public Builder(Context context) {
            this.context = context;
        }

        public Builder setUrl(String url) {
            this.url = url;
            return this;
        }

        public Builder setUrl(int resId) {
            this.url = (String) context.getText(resId);
            return this;
        }

        public Builder setWidth(int width) {
            this.width = width;
            return this;
        }

        public Builder setHeight(int height) {
            this.height = height;
            return this;
        }

        public Builder setPositiveButton(OnClickListener listener) {
            this.positiveButtonClickListener = listener;
            return this;
        }

        public Builder setCententLayoutId(int layoutId) {
            this.layoutId = layoutId;
            return this;
        }

        public Builder setImageReadyListener(OnImageReadyCallback callback) {
            this.imageReadyCallback = callback;
            return this;
        }

        public PublicPictureDialog create() {
            // instantiate the dialog with the custom Theme
            final PublicPictureDialog dialog = new PublicPictureDialog(context, R.style.confirm_dialog);
            View layout = LayoutInflater.from(context).inflate(layoutId, null);
            ImageView imageView = layout.findViewById(R.id.ivTipContent);
            FrameLayout containerView=layout.findViewById(R.id.flContainerView);

            Window dialogWindow = dialog.getWindow();
            if (null != dialogWindow) {
                dialogWindow.setGravity(Gravity.CENTER);
            }

            if (width > 0 || height > 0) {
                containerView.setLayoutParams(new LinearLayout.LayoutParams(width, height));
            }

            if (imageReadyCallback != null) {
                imageReadyCallback.onImageReady(dialog, url, imageView);
            } else {
                ImageView ivRefreshView = layout.findViewById(R.id.ivRefresh);
                rotateAnimation = rotateAlways();
                ivRefreshView.setOnClickListener(v -> {
                    ivRefreshView.startAnimation(rotateAnimation);
                    v.postDelayed(() -> loadImageUrlToView(imageView, ivRefreshView), 2000);
                });

                loadImageUrlToView(imageView, ivRefreshView);
            }


            // set the confirm button
            if (positiveButtonClickListener != null) {
                layout.findViewById(R.id.btnTipCancel)
                        .setOnClickListener(v -> positiveButtonClickListener.onClick(dialog,
                                DialogInterface.BUTTON_POSITIVE));
            }
            dialog.addContentView(layout, new LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT));
            return dialog;
        }

        /**
         * 加载图片
         */
        private void loadImageUrlToView(ImageView imageView, ImageView ivRefreshView) {
            Glide.with(context)
                    .load(!TextUtils.isEmpty(url) ? url : R.drawable.alt)
                    .dontAnimate()//防止设置placeholder导致第一次不显示网络图片,只显示默认图片的问题
                    .error(R.drawable.alt)
                    .placeholder(R.drawable.alt)
                    .listener(new RequestListener<Serializable, GlideDrawable>() {
                        @Override
                        public boolean onException(Exception e, Serializable serializable, Target<GlideDrawable> target, boolean b) {
                            if (refreshCount < 3) {
                                loadImageUrlToView(imageView, ivRefreshView);
                            } else {
                                ivRefreshView.clearAnimation();
                                ivRefreshView.setVisibility(View.VISIBLE);
                            }
                            refreshCount++;
                            return true;
                        }

                        @Override
                        public boolean onResourceReady(GlideDrawable glideDrawable, Serializable serializable, Target<GlideDrawable> target, boolean b, boolean b1) {
                            refreshCount = 0;
                            ivRefreshView.clearAnimation();
                            ivRefreshView.setVisibility(View.GONE);
                            return false;
                        }
                    })
                    .into(imageView);

        }
    }

    public static RotateAnimation rotateAlways() {
        RotateAnimation rotateAnimation = new RotateAnimation(0, 360,
                Animation.RELATIVE_TO_SELF, 0.5f, Animation.RELATIVE_TO_SELF, 0.5f);
        rotateAnimation.setDuration(500);
        rotateAnimation.setFillAfter(true);
        rotateAnimation.setRepeatCount(-1);
        rotateAnimation.setInterpolator(new LinearInterpolator());
        return rotateAnimation;
    }
}